@@ -4,7 +4,7 @@
  * See the main source file 'xineliboutput.c' for copyright information and
  * how to reach the author.
  *
- * $Id: device.h,v 1.63 2012/03/17 20:14:41 phintuka Exp $
+ * $Id$
  *
  */
 
@@ -90,6 +90,7 @@ class cXinelibDevice : public cDevice
     virtual bool HasDecoder(void) const { return true; };
     virtual bool CanReplay(void) const { return true; };
     virtual bool HasIBPTrickSpeed(void) { return xc.ibp_trickspeed; }
+    virtual cRect CanScaleVideo(const cRect &Rect, int Alignment = taCenter);
 
     bool SupportsTrueColorOSD(void);
 
@@ -99,35 +100,36 @@ class cXinelibDevice : public cDevice
     ePlayMode m_PlayMode;
     int       m_TrickSpeed;
     int       m_TrickSpeedMode;
-#if VDRVERSNUM < 10705
-    int64_t   m_TrickSpeedPts;
-    int       m_TrickSpeedDelay;
-#endif
+    cRect     m_VideoWindow;
 
   public:
     virtual bool SetPlayMode(ePlayMode PlayMode);
     ePlayMode GetPlayMode(void) const { return m_PlayMode; };
 
+    const cRect& GetVideoWindow(void) { return m_VideoWindow; }
+
   protected:
     virtual void    Clear(void);
     virtual void    Play(void);
+    virtual void    TrickSpeed(int Speed, bool Forward) { TrickSpeed(Speed); }
     virtual void    TrickSpeed(int Speed);
     virtual void    Freeze(void);
     virtual bool    Flush(int TimeoutMs = 0);
     virtual int64_t GetSTC(void);
+    virtual void    ScaleVideo(const cRect &Rect = cRect::Null);
 
   // Video format facilities
 
   public:
     virtual void SetVideoDisplayFormat(eVideoDisplayFormat VideoDisplayFormat);
     virtual void SetVideoFormat(bool VideoFormat16_9);
+#if VDRVERSNUM < 20301
     virtual eVideoSystem GetVideoSystem(void);
+#endif
 
     struct video_size_s *m_VideoSize;
     struct ts_state_s   *m_tssVideoSize;
-#if VDRVERSNUM >= 10708
     virtual void GetVideoSize(int &Width, int &Height, double &VideoAspect);
-#endif
     virtual void GetOsdSize(int &Width, int &Height, double &PixelAspect);
 
   // Track facilities
@@ -170,8 +172,9 @@ class cXinelibDevice : public cDevice
     cXinelibStatusMonitor *m_statusMonitor;
     bool m_liveMode;
 
-  public:
-    void SetTvMode(cChannel *Channel);
+  protected:
+    friend class cXinelibStatusMonitor;
+    void SetTvMode(int ChannelNumber);
     void SetReplayMode(void);
     void StopOutput(void);
 
@@ -237,7 +240,6 @@ class cXinelibDevice : public cDevice
     int  m_FreeBufs;
 
     int PlayAny(const uchar *Data, int Length);
-    int PlayTrickSpeed(const uchar *buf, int length);
 
     bool AcceptVideoPacket(const uchar *Data, int Length);
     bool AcceptAudioPacket(const uchar *Data, int Length);
@@ -253,7 +255,6 @@ class cXinelibDevice : public cDevice
     virtual int  PlayAudio(const uchar *Data, int Length, uchar Id);
     virtual int  PlaySubtitle(const uchar *Data, int Length);
 
-#if VDRVERSNUM >= 10701 || defined(TSPLAY_PATCH_VERSION)
     // join multiple TS packets to xineliboutput transport packet
     cMutex        m_TsBufLock;
     uint8_t       m_TsBuf[4096];
@@ -269,10 +270,6 @@ class cXinelibDevice : public cDevice
     virtual int PlayTsAudio(const uchar *Data, int Length);
     virtual int PlayTsSubtitle(const uchar *Data, int Length);
     virtual int PlayTs(const uchar *Data, int Length, bool VideoOnly = false);
-#else
-    void        TsBufferClear(void) {}
-    void        TsBufferFlush(void) {}
-#endif
 
   // Picture-In-Picture
 
