@@ -4,7 +4,7 @@
  * See the main source file 'xineliboutput.c' for copyright information and
  * how to reach the author.
  *
- * $Id: frontend.c,v 1.59.2.2 2008/09/26 13:46:49 phintuka Exp $
+ * $Id$
  *
  */
 
@@ -28,6 +28,7 @@
 #include <vdr/config.h>
 #include <vdr/tools.h>
 #include <vdr/plugin.h>
+#include <vdr/device.h>
 
 #include "logdefs.h"
 #include "config.h"
@@ -35,11 +36,11 @@
 #include "device.h"
 
 #include "tools/pes.h"
+#include "tools/mpeg.h"
+#include "tools/h264.h"
 #include "tools/general_remote.h"
 #include "tools/iso639.h"
 
-//#define LOG_CONTROL_MESSAGES
-//#define XINELIBOUTPUT_LOG_KEYS
 
 #ifndef STARTUP_IMAGE_FILE
 #  define  STARTUP_IMAGE_FILE "/usr/share/vdr/xineliboutput/logo.mpv"
@@ -55,36 +56,51 @@
 //
 
 /*static*/
-void cXinelibThread::KeypressHandler(const char *keymap, const char *key, 
-				     bool repeat, bool release)
+void cXinelibThread::KeypressHandler(const char *keymap, const char *key,
+                                     bool repeat, bool release)
 {
-#ifdef XINELIBOUTPUT_LOG_KEYS
-  static FILE *flog = fopen("/video/keys.log","w");
-  if (flog) {
-    fprintf(flog,"KEY %s %s %d %d\n",keymap,key,repeat,release); fflush(flog);
-  }
-#endif
-
   TRACE("keypress_handler: " << (keymap?keymap:"") << " " << key);
 
-  if(!key)
+  // check if key exists.
+  // Note: empty key ("") is used to trigger learning; it only creates the cRemote object.
+  if (!key)
     return;
 
-  if(keymap) {
-    cRemote *item = Remotes.First();
-    while(item) {
-      if(!strcmp(item->Name(), keymap)) {
-	// dirty... but only way to support learning ...
-	((cGeneralRemote*)item)->Put(key, repeat, release);
-	return;
-      }
-      item = Remotes.Next(item);
-    }
-    cGeneralRemote *r = new cGeneralRemote(keymap);
-    if(*key)
-      r->Put(key, repeat, release);
-  } else {
+  if (!keymap) {
+    // raw VDR key
     cRemote::Put(cKey::FromString(key));
+    return;
+  }
+
+  // find correct remote
+  cGeneralRemote *remote = NULL;
+  for (cRemote *item = Remotes.First(); item; item = Remotes.Next(item)) {
+    if (!strcmp(item->Name(), keymap)) {
+      // dirty... but using protected cRemote::Put() is the only way to support learning ...
+      remote = (cGeneralRemote*)item;
+      break;
+    }
+  }
+
+  // not found ? create new one
+  if (!remote)
+    remote = new cGeneralRemote(keymap);
+
+  // put key to remote queue
+  if (key[0]) {
+    if (!remote->Put(key, repeat, release)) {
+      if (!strcmp(keymap, "KBD")) {
+        uint64_t value = 0;
+        sscanf(key, "%" PRIX64, &value);
+        if (value) {
+          remote->cRemote::Put(KBDKEY(value));
+          return;
+        }
+      }
+      if (!key[1]) {
+        remote->cRemote::Put(KBDKEY(key[0]));
+      }
+    }
   }
 }
 
@@ -107,9 +123,6 @@ void cXinelibThread::InfoHandler(const char *info)
   if(!strncmp(info, "TRACKMAP SPU", 12)) {
     int CurrentTrack = ttXSubtitleAuto;
     map += 12;
-#if VDRVERSNUM < 10515 && !defined(VDRSPUPATCH)
-    cXinelibDevice::Instance().ClrAvailableDvdSpuTracks(false);
-#endif
     while(*map) {
       bool Current = false;
       while(*map == ' ') map++;
@@ -129,30 +142,22 @@ void cXinelibThread::InfoHandler(const char *info)
 	char *lang = map;
 	while(*map && *map != ' ') map++;
 	if(*map == ' ') { *map = 0; map++; };
-#if VDRVERSNUM < 10515 && !defined(VDRSPUPATCH)
-	cXinelibDevice::Instance().SetAvailableDvdSpuTrack(id, iso639_2_to_iso639_1(lang), Current);
-#else
-	cXinelibDevice::Instance().SetAvailableTrack(ttSubtitle, id, id+1, iso639_2_to_iso639_1(lang));
-	if (Current) 
+	m_Dev->SetAvailableTrack(ttSubtitle, id, id+1, iso639_1_to_iso639_2(lang) ?: *cString::sprintf("%03d", id+1));
+	if (Current)
 	  CurrentTrack = id;
-#endif
       }
     }
-#if VDRVERSNUM < 10515 && !defined(VDRSPUPATCH)
-    cXinelibDevice::Instance().EnsureDvdSpuTrack();
-#else
     if (CurrentTrack == ttXSubtitleAuto)
-      cXinelibDevice::Instance().EnsureSubtitleTrack();
+      m_Dev->EnsureSubtitleTrack();
     else if (CurrentTrack == ttXSubtitleNone)
-      cXinelibDevice::Instance().SetCurrentSubtitleTrack(ttNone, true);
+      m_Dev->SetCurrentSubtitleTrack(ttNone, true);
     else
-      cXinelibDevice::Instance().SetCurrentSubtitleTrack(eTrackType(CurrentTrack+ttSubtitleFirst), true);
-#endif
+      m_Dev->SetCurrentSubtitleTrack(eTrackType(CurrentTrack+ttSubtitleFirst), true);
   }
 
   else if(!strncmp(info, "TRACKMAP AUDIO", 14)) {
     map += 14;
-    cXinelibDevice::Instance().ClrAvailableTracks();
+    m_Dev->ClrAvailableTracks();
     while(*map) {
       bool Current = false;
       while(*map == ' ') map++;
@@ -166,9 +171,9 @@ void cXinelibThread::InfoHandler(const char *info)
       char *lang = map;
       while(*map && *map != ' ') map++;
       if(*map == ' ') { *map = 0; map++; };
-      cXinelibDevice::Instance().SetAvailableTrack(ttDolby, id, ttDolby+id, iso639_2_to_iso639_1(lang));
-      if(Current) 
-	cXinelibDevice::Instance().SetCurrentAudioTrack((eTrackType)(ttDolby+id));
+      m_Dev->SetAvailableTrack(ttDolby, id, ttDolby+id, iso639_1_to_iso639_2(lang) ?: *cString::sprintf("%03d", id+1));
+      if(Current)
+	m_Dev->SetCurrentAudioTrack((eTrackType)(ttDolby+id));
     }
   }
 
@@ -187,123 +192,92 @@ void cXinelibThread::InfoHandler(const char *info)
       *end = 0;
 
       if(!strcmp(map, "title"))
-	cXinelibDevice::Instance().SetMetaInfo(miTitle, next);
+	m_Dev->SetMetaInfo(miTitle, next);
       if(!strcmp(map, "tracknumber"))
-        cXinelibDevice::Instance().SetMetaInfo(miTracknumber, next);
+        m_Dev->SetMetaInfo(miTracknumber, next);
       if(!strcmp(map, "album"))
-	cXinelibDevice::Instance().SetMetaInfo(miAlbum, next);
+	m_Dev->SetMetaInfo(miAlbum, next);
       if(!strcmp(map, "artist"))
-	cXinelibDevice::Instance().SetMetaInfo(miArtist, next);
+	m_Dev->SetMetaInfo(miArtist, next);
       map = end+1;
     }
   }
-   
+
   else if(!strncmp(info, "DVDBUTTONS ", 11)) {
     map += 11;
     while(*map == ' ') map++;
-    cXinelibDevice::Instance().SetMetaInfo(miDvdButtons, map);
+    m_Dev->SetMetaInfo(miDvdButtons, map);
   }
 
   else if(!strncmp(info, "TITLE ", 6)) {
     map += 6;
     while(*map == ' ') map++;
-    cXinelibDevice::Instance().SetMetaInfo(miTitle, map);
+    m_Dev->SetMetaInfo(miTitle, map);
   }
 
   else if(!strncmp(info, "DVDTITLE ", 9)) {
-    LOGMSG("DVDTITLE %s", info);
     map += 9;
     while(*map == ' ') map++;
-    cXinelibDevice::Instance().SetMetaInfo(miDvdTitleNo, map);
-#if VDRVERSNUM < 10515 && !defined(VDRSPUPATCH)
+    m_Dev->SetMetaInfo(miDvdTitleNo, map);
     if (*map == '0')  // DVD Menu, set spu track to 0
-      cXinelibDevice::Instance().SetCurrentDvdSpuTrack(0);
-#else
-    if (*map == '0')  // DVD Menu, set spu track to 0
-      cXinelibDevice::Instance().SetCurrentSubtitleTrack(ttSubtitleFirst);
-#endif
+      m_Dev->SetCurrentSubtitleTrack(ttSubtitleFirst);
+  }
+
+  else if (!strncmp(info, "WINDOW ", 7)) {
+    int w, h;
+    map += 7;
+    while(*map == ' ') map++;
+    if (2 == sscanf(map, "%dx%d", &w, &h)) {
+      xc.osd_width_auto  = w;
+      xc.osd_height_auto = h;
+    }
   }
 
   free(pmap);
 }
 
-cXinelibThread::cXinelibThread(const char *Description) : cThread(Description)
+cXinelibThread::cXinelibThread(cXinelibDevice *Dev, const char *Description) : cThread(Description)
 {
   TRACEF("cXinelibThread::cXinelibThread");
 
+  m_Dev = Dev;
+
   m_Volume = 255;
-  m_bStopThread = false;
   m_bReady = false;
-  m_bIsFinished = false;
   m_bNoVideo = true;
+  m_bHDMode = false;
   m_bLiveMode = true; /* can't be replaying when there is no output device */
   m_StreamPos = 0;
+  m_LastClearPos = 0;
   m_Frames = 0;
+  m_TrickSpeed = -1;
+  m_bTrickSpeedBack = false;
   m_bEndOfStreamReached = false;
   m_bPlayingFile = false;
   m_StatusMonitor = NULL;
+  m_SpuLangAuto = true;
 }
 
-cXinelibThread::~cXinelibThread() 
+cXinelibThread::~cXinelibThread()
 {
   TRACEF("cXinelibThread::~cXinelibThread");
 
-  m_bStopThread = true;
-  if(Active())
-    Cancel();
-  if(m_StatusMonitor)
-    delete m_StatusMonitor;
+  Cancel(3);
+
+  if (m_StatusMonitor)
+    DELETENULL(m_StatusMonitor);
 }
 
 //
 // Thread control
 //
 
-void cXinelibThread::Start(void) 
-{
-  TRACEF("cXinelibThread::Start");
-
-  cThread::Start(); 
-}
-
-void cXinelibThread::Stop(void)  
-{ 
-  TRACEF("cXinelibThread::Stop");
-
-  SetStopSignal(); 
-
-  //if(Active()) 
-  Cancel(5); 
-}
-
-void cXinelibThread::SetStopSignal(void) 
-{
-  TRACEF("cXinelibThread::SetStopSignal");
-
-  LOCK_THREAD;
-  m_bStopThread = true;
-}
-
-bool cXinelibThread::GetStopSignal(void) 
-{
-  TRACEF("cXinelibThread::GetStopSignal");
-
-  LOCK_THREAD;
-  return m_bStopThread;
-}
-
 bool cXinelibThread::IsReady(void)
 {
   LOCK_THREAD;
   return m_bReady;
 }
 
-bool cXinelibThread::IsFinished(void)
-{
-  LOCK_THREAD;
-  return m_bIsFinished;
-}
-
 //
 // Playback control
 //
@@ -311,16 +285,22 @@ bool cXinelibThread::IsFinished(void)
 void cXinelibThread::SetVolume(int NewVolume)
 {
   m_Volume = NewVolume;
-  cString str = cString::sprintf("VOLUME %d%s", NewVolume * 100 / 255, 
-				 xc.sw_volume_control ? " SW" : "");
+  cString str = cString::sprintf("VOLUME %d%s", NewVolume * 100 / 255,
+                                 xc.sw_volume_control ? " SW" : "");
   Xine_Control(str);
 }
 
-void cXinelibThread::TrickSpeed(int Speed)
+void cXinelibThread::TrickSpeed(int Speed, bool Backwards)
 {
   TRACEF("cXinelibThread::TrickSpeed");
 
-  Xine_Control("TRICKSPEED", Speed);
+  Lock();
+  m_TrickSpeed      = Speed;
+  m_bTrickSpeedBack = Backwards;
+  Unlock();
+
+  cString str = cString::sprintf("TRICKSPEED %d%s", Speed, Backwards ? " Backwards" : "");
+  Xine_Control(str);
 }
 
 void cXinelibThread::SetLiveMode(bool LiveModeOn)
@@ -358,12 +338,14 @@ void cXinelibThread::SetNoVideo(bool bVal)
 
   Xine_Control("NOVIDEO", m_bNoVideo ? 1 : 0);
 
-  char *opts = NULL;
-  if(xc.audio_vis_goom_opts[0] && !strcmp(xc.audio_visualization, "goom"))
-    opts = xc.audio_vis_goom_opts;
+  if(m_bNoVideo && strcmp(xc.audio_visualization, "none") && strcmp(xc.audio_visualization, "image")) {
+
+    char *opts = NULL;
+    if(xc.audio_vis_goom_opts[0] && !strcmp(xc.audio_visualization, "goom"))
+      opts = xc.audio_vis_goom_opts;
 
-  if(m_bNoVideo && strcmp(xc.audio_visualization, "none")) {
     ConfigurePostprocessing(xc.audio_visualization, true, opts);
+
   } else {
     ConfigurePostprocessing("AudioVisualization", false, NULL);
   }
@@ -378,13 +360,6 @@ void cXinelibThread::AudioStreamChanged(bool ac3, int StreamId)
     Xine_Control("AUDIOSTREAM", StreamId);
 }
 
-#if VDRVERSNUM < 10515
-void cXinelibThread::SpuStreamChanged(int StreamId)
-{
-  TRACEF("cXinelibThread::SpuStreamChanged");
-  Xine_Control("SPUSTREAM", StreamId);
-}
-#else
 void cXinelibThread::SetSubtitleTrack(eTrackType Track)
 {
   TRACEF("cXinelibThread::SetSubtitleTrack");
@@ -393,20 +368,36 @@ void cXinelibThread::SetSubtitleTrack(eTrackType Track)
 				 m_SpuLangAuto ? " auto" : "");
   Xine_Control(buf);
 }
-#endif
+
+void cXinelibThread::Pip_Config(int Index, int X, int Y, int W, int H)
+{
+  Xine_Control(cString::sprintf("PIP %d %d %d %d %d", Index, X, Y, W, H));
+}
+
+void cXinelibThread::Pip_Close(int Index)
+{
+  Xine_Control(cString::sprintf("PIP %d Close", Index));
+}
 
 void cXinelibThread::Clear(void)
 {
   TRACEF("cXinelibThread::Clear");
 
-  Lock();
-  int64_t  tmp1 = m_StreamPos;
-  uint32_t tmp2 = m_Frames;
-  Unlock();
-
   char buf[128];
-  snprintf(buf, sizeof(buf), "DISCARD %" PRId64 " %d", tmp1, tmp2);
-  /* Send to control stream and data stream. If message is sent only to 
+
+  {
+    LOCK_THREAD;
+
+    if (m_StreamPos == m_LastClearPos) {
+      //LOGDBG("cXinelibThread::Clear(): double Clear() ignored");
+      return;
+    }
+    m_LastClearPos = m_StreamPos;
+
+    snprintf(buf, sizeof(buf), "DISCARD %" PRId64 " %d", m_StreamPos, m_Frames);
+  }
+
+  /* Send to control stream and data stream. If message is sent only to
    * control stream, and it is delayed, engine flush will be skipped.
    */
   Xine_Control(buf);
@@ -440,13 +431,14 @@ int cXinelibThread::Poll(cPoller& Poller, int TimeoutMs)
 // Data transfer
 //
 
-int cXinelibThread::Play_PES(const uchar *data, int len)
+int cXinelibThread::Play(const uchar *data, int len, eStreamId StreamId)
 {
-  Lock();
-  m_StreamPos += len;
-  m_Frames++;
-  /*m_bEndOfStreamReached = false;*/
-  Unlock();
+  if (StreamId == sidVdr && len >= 0) {
+    Lock();
+    m_StreamPos += len;
+    m_Frames++;
+    Unlock();
+  }
   return len;
 }
 
@@ -481,8 +473,11 @@ int cXinelibThread::Play_Mpeg1_PES(const uchar *data1, int len)
       i1 += 2;
     }
     
-    if(len<i1+5) return len;
-    
+    if (len < i1 + 5) {
+      delete[] data2;
+      return len;
+    }
+
     data2[i2++] = 0x80;
     
     if ((data1[i1] & 0xf0) == 0x20) { 
@@ -525,10 +520,10 @@ int cXinelibThread::Play_Mpeg1_PES(const uchar *data1, int len)
       while(!Poll(p,100) && loops++ < 10) {
 	LOGDBG("Play_Mpeg1_PES: poll failed");
       }
-      r = Play_PES(data2,newlen+6);
+      r = Play(data2, newlen + 6);
     } 
   
-    delete data2;
+    delete [] data2;
     return r==newlen+6 ? ((data1[4]<<8)|data1[5])+6 : 0;
   }
   return len; // nothing useful found ...
@@ -536,22 +531,21 @@ int cXinelibThread::Play_Mpeg1_PES(const uchar *data1, int len)
 
 // Pack elementary MPEG stream to PES
 
-bool cXinelibThread::Play_Mpeg2_ES(const uchar *data, int len, int streamID)
+bool cXinelibThread::Play_Mpeg2_ES(const uchar *data, int len, int streamID, bool h264)
 {
-  static uchar hdr_vid[] = {0x00,0x00,0x01,0xe0, 0x00,0x00,0x80,0x00,0x00}; /* mpeg2 */
-  static uchar hdr_pts[] = {0x00,0x00,0x01,0xe0, 0x00,0x08,0x80,0x80,
-                            0x05,0x00,0x00,0x00, 0x00,0x00}; /* mpeg2 */
-  static uchar seq_end[] = {0x00,0x00,0x01,0xe0, 0x00,0x07,0x80,0x00,
-			    0x00,
-			    0x00,0x00,0x01,0xB7}; /* mpeg2 */
+  uchar hdr_vid[] = {0x00,0x00,0x01,0xe0, 0x00,0x00,0x80,0x00,0x00}; /* mpeg2 */
+  uchar hdr_pts[] = {0x00,0x00,0x01,0xe0, 0x00,0x08,0x80,0x80,
+                     0x05,0x00,0x00,0x00, 0x00,0x00}; /* mpeg2 */
+  uchar seq_end[] = {0x00,0x00,0x01,0xe0, 0x00,0x07,0x80,0x00,
+                     0x00,
+                     0x00,0x00,0x01,SC_SEQUENCE_END}; /* mpeg2 */
   int todo = len, done = 0, hdrlen = 9/*sizeof(hdr)*/;
   uchar *frame = new uchar[PES_CHUNK_SIZE+32];
   cPoller p;
 
-
   hdr_pts[3] = (uchar)streamID;
   Poll(p, 100);
-  Play_PES(hdr_pts, sizeof(hdr_pts));
+  Play(hdr_pts, sizeof(hdr_pts));
 
   hdr_vid[3] = (uchar)streamID;
   while(todo) {
@@ -569,8 +563,8 @@ bool cXinelibThread::Play_Mpeg2_ES(const uchar *data, int len, int streamID)
 
     Poll(p, 100);
 
-    if(blocklen+hdrlen != Play_PES(frame,blocklen+hdrlen)) {      
-      delete frame;
+    if (blocklen + hdrlen != Play(frame, blocklen + hdrlen)) {
+      delete [] frame;
       return false;
     }
   }
@@ -578,8 +572,9 @@ bool cXinelibThread::Play_Mpeg2_ES(const uchar *data, int len, int streamID)
   // append sequence end code to video 
   if((streamID & 0xF0) == 0xE0) { 
     seq_end[3] = (uchar)streamID;
+    seq_end[12] = h264 ? NAL_END_SEQ : SC_SEQUENCE_END;
     Poll(p, 100);
-    Play_PES(seq_end, sizeof(seq_end));
+    Play(seq_end, sizeof(seq_end));
   }
 
   delete[] frame;
@@ -593,12 +588,6 @@ bool cXinelibThread::Play_Mpeg2_ES(const uchar *data, int len, int streamID)
 bool cXinelibThread::QueueBlankDisplay(void)
 {
   TRACEF("cXinelibThread::BlankDisplay");
-#if 0
-  extern const unsigned char v_mpg_black[];     // black_720x576.c
-  extern const int v_mpg_black_length;
-
-  Play_Mpeg2_ES(v_mpg_black, v_mpg_black_length, VIDEO_STREAM);
-#endif
   Xine_Control_Sync("BLANK");
   return true;
 }
@@ -622,9 +611,9 @@ bool cXinelibThread::LogoDisplay(void)
 
   if(Setup.FileName()) {
     cString SetupPath = Setup.FileName();
-    char *end = strrchr(SetupPath, '/');
+    const char *end = strrchr(SetupPath, '/');
     if(end) {
-      *end = 0;
+      SetupPath.Truncate(end - (const char *)SetupPath);
       fd = open(Path=cString::sprintf("%s/plugins/xineliboutput/logo.mpv", *SetupPath), O_RDONLY);
     }
   }
@@ -635,6 +624,7 @@ bool cXinelibThread::LogoDisplay(void)
   if(fd >= 0) {
     uint8_t *data = (uint8_t*)malloc(STARTUP_MAX_SIZE);
     int datalen = read(fd, data, STARTUP_MAX_SIZE);
+    close(fd);
     if(datalen == STARTUP_MAX_SIZE) {
       LOGMSG("WARNING: custom startup image %s too large", *Path);
     } else if(datalen<=0) {
@@ -648,7 +638,6 @@ bool cXinelibThread::LogoDisplay(void)
       return r;
     }
     free(data);
-    close(fd);
   }
   
   /* use default image */
@@ -711,61 +700,60 @@ int cXinelibThread::Xine_Control(const char *cmd, const char *p1)
   return Xine_Control(buf);
 }
 
-bool cXinelibThread::PlayFile(const char *FileName, int Position, 
-			      bool LoopPlay, ePlayMode PlayMode)
+bool cXinelibThread::PlayFile(const char *FileName, int Position,
+                              bool LoopPlay, ePlayMode PlayMode,
+                              int TimeoutMs)
 {
   TRACEF("cXinelibThread::PlayFile");
 
-  char vis[256];
+  cString vis, buf;
 
-  switch(PlayMode) {
+  switch (PlayMode) {
     case pmVideoOnly:
       LOGDBG("cXinelibThread::PlayFile: Video from file, audio from VDR");
-      strcpy(vis, "Video");
+      vis = "Video";
       break;
     case pmAudioOnly:
       LOGDBG("cXinelibThread::PlayFile: Audio from file, video from VDR");
-      strcpy(vis, "Audio");
+      vis = "Audio";
       break;
     case pmAudioOnlyBlack:
       //LOGDBG("cXinelibThread::PlayFile: Audio from file, no video");
-      strcpy(vis, "none");
+      vis = "none";
       break;
     case pmAudioVideo:
     default:
-      if(xc.audio_vis_goom_opts[0] && !strcmp(xc.audio_visualization, "goom"))
-	snprintf(vis, sizeof(vis), "%s:%s", xc.audio_visualization, xc.audio_vis_goom_opts);
-      else
-	strn0cpy(vis, xc.audio_visualization, sizeof(vis));
-      vis[sizeof(vis)-1] = 0;
+      if (xc.audio_vis_goom_opts[0] && !strcmp(xc.audio_visualization, "goom")) {
+	vis = cString::sprintf("%s:%s", xc.audio_visualization, xc.audio_vis_goom_opts);
+      } else if (xc.audio_vis_image_mrl[0] && !strcmp(xc.audio_visualization, "image")) {
+	vis = cString::sprintf("%s:%s", xc.audio_visualization, xc.audio_vis_image_mrl);
+      } else {
+	vis = xc.audio_visualization;
+      }
       break;
   }
 
-  char buf[4096];
   m_bEndOfStreamReached = false;
-  if(snprintf(buf, sizeof(buf), "PLAYFILE %s %d %s %s",
-	      LoopPlay ? "Loop" : "", Position, vis, FileName ? FileName : "")
-     >= 4096) {
-    LOGMSG("PlayFile: message too long !");
-    return 0;
-  }
+  buf = cString::sprintf("PLAYFILE %s %d %s %s",
+                         LoopPlay ? "Loop" : "",
+                         Position,
+                         *vis,
+                         FileName ?: "");
 
-  if(FileName) {
+  if (FileName) {
     Lock();
     m_FileName = FileName;
     m_bPlayingFile = true;
     m_SpuLangAuto = true;
-#if VDRVERSNUM >= 10515
     if (m_StatusMonitor)
       DELETENULL(m_StatusMonitor);
     m_StatusMonitor = new cFrontendStatusMonitor(m_SpuLangAuto);
-#endif
     Unlock();
   }
 
-  int result = PlayFileCtrl(buf);
+  int result = PlayFileCtrl(buf, TimeoutMs);
 
-  if(!FileName || result != 0) {
+  if (!FileName || result != 0) {
     Lock();
     m_bPlayingFile = false;
     m_FileName = NULL;
@@ -773,18 +761,18 @@ bool cXinelibThread::PlayFile(const char *FileName, int Position,
       DELETENULL(m_StatusMonitor);
     Unlock();
   } else {
-    if(xc.extsub_size >= 0)
+    if (xc.extsub_size >= 0)
       Xine_Control("EXTSUBSIZE", xc.extsub_size);
 
-#if VDRVERSNUM >= 10515
-    // set preferred subtitle language 
+    // set preferred subtitle language
     if (Setup.DisplaySubtitles) {
       const char *langs = I18nLanguageCode(Setup.SubtitleLanguages[0]);
       if (langs) {
 	char lang1[5];
 	strn0cpy(lang1, langs, 4); /* truncate */
-	const char *spu_lang = iso639_1_to_iso639_2(lang1);
+	const char *spu_lang = iso639_2_to_iso639_1(lang1);
 	LOGMSG("Preferred SPU language: %s (%s)", lang1, spu_lang);
+        Xine_Control(cString::sprintf("SPUSTREAM %s", lang1));
 	if (spu_lang && spu_lang[0] && spu_lang[1] && !spu_lang[2])
 	  Xine_Control(cString::sprintf("SPUSTREAM %s", spu_lang));
       }
@@ -792,10 +780,9 @@ bool cXinelibThread::PlayFile(const char *FileName, int Position,
       LOGMSG("Preferred SPU language: (none)");
       Xine_Control(cString::sprintf("SPUSTREAM %d", ttXSubtitleNone));
     }
-#endif
   }
 
-  return (!GetStopSignal()) && (result==0);
+  return Running() && !result;
 }
 
 
@@ -808,7 +795,7 @@ void cXinelibThread::Configure(void)
     ConfigurePostprocessing(xc.deinterlace_method, xc.audio_delay, 
 			    xc.audio_compression, xc.audio_equalizer,
 			    xc.audio_surround, xc.speaker_type);
-    ConfigureVideo(xc.hue, xc.saturation, xc.brightness, xc.contrast, xc.overscan, xc.vo_aspect_ratio);
+    ConfigureVideo(xc.hue, xc.saturation, xc.brightness, xc.sharpness, xc.noise_reduction, xc.contrast, xc.overscan, xc.vo_aspect_ratio);
     ConfigurePostprocessing("upmix",     xc.audio_upmix ? true : false, NULL);
     ConfigurePostprocessing("autocrop",  xc.autocrop  ? true : false, 
 			    xc.AutocropOptions());
@@ -827,7 +814,10 @@ void cXinelibThread::Configure(void)
 
     Xine_Control(cString::sprintf("SCR %s %d", 
 				  xc.live_mode_sync ? "Sync"    : "NoSync",
-				  xc.scr_tunning    ? xc.scr_hz : 90000));
+				  xc.scr_tuning     ? xc.scr_hz : 90000));
+    Xine_Control("HDMODE", m_bHDMode);
+
+    Xine_Control("CONFIG END");
 }
 
 int cXinelibThread::ConfigurePostprocessing(const char *deinterlace_method, 
@@ -837,7 +827,6 @@ int cXinelibThread::ConfigurePostprocessing(const char *deinterlace_method,
 					    int audio_surround,
 					    int speaker_type) 
 {
-  char buf[1024];
   int r = true;
 
   if(strcmp(deinterlace_method, "tvtime")) 
@@ -848,15 +837,14 @@ int cXinelibThread::ConfigurePostprocessing(const char *deinterlace_method,
   r = Xine_Control("AUDIOCOMPRESSION", audio_compression) && r;
   r = Xine_Control("AUDIOSURROUND", audio_surround) && r;
   r = Xine_Control("SPEAKERS", speaker_type) && r;
-  sprintf(buf,"EQUALIZER %d %d %d %d %d %d %d %d %d %d",
-	  audio_equalizer[0],audio_equalizer[1],
-	  audio_equalizer[2],audio_equalizer[3],
-	  audio_equalizer[4],audio_equalizer[5],
-	  audio_equalizer[6],audio_equalizer[7],
-	  audio_equalizer[8],audio_equalizer[9]);
-  r = Xine_Control(buf) && r;
+  r = Xine_Control(cString::sprintf("EQUALIZER %d %d %d %d %d %d %d %d %d %d",
+                                    audio_equalizer[0], audio_equalizer[1],
+                                    audio_equalizer[2], audio_equalizer[3],
+                                    audio_equalizer[4], audio_equalizer[5],
+                                    audio_equalizer[6], audio_equalizer[7],
+                                    audio_equalizer[8], audio_equalizer[9])) && r;
 
-  if(m_bNoVideo && strcmp(xc.audio_visualization, "none")) {
+  if (m_bNoVideo && strcmp(xc.audio_visualization, "none") && strcmp(xc.audio_visualization, "image")) {
     char *opts = NULL;
     if(xc.audio_vis_goom_opts[0] && !strcmp(xc.audio_visualization, "goom"))
       opts = xc.audio_vis_goom_opts;
@@ -875,37 +863,22 @@ int cXinelibThread::ConfigurePostprocessing(const char *deinterlace_method,
 
 int cXinelibThread::ConfigurePostprocessing(const char *name, bool on, const char *args)
 {
-  char buf[1024];  
-  int l;
-
-  if(on) 
-    l = snprintf(buf, sizeof(buf), "POST %s On %s", (name&&*name)?name:"0", args?args:"");
-  else 
-    // 0 - audio vis.
-    // 1 - audio post
-    // 2 - video post
-    //return fe->post_close(fe, name, -1);
-    l = snprintf(buf, sizeof(buf), "POST %s Off", (name&&*name)?name:"0");
-
-  if(l >= (int)sizeof(buf)) {
-    LOGMSG("ConfigurePostprocessing %s: message too long !", name);
-    return 0;
-  }
-  //buf[sizeof(buf)-1] = 0;
-
-  return Xine_Control(buf);
+  cString cmd = cString::sprintf("POST %s %s %s",
+                                 (name && *name) ? name : "0",
+                                 on ? "On" : "Off",
+                                 (on && args) ? args : "");
+  return Xine_Control(cmd);
 }
 
-int cXinelibThread::ConfigureVideo(int hue, int saturation, 
-				   int brightness, int contrast,
+int cXinelibThread::ConfigureVideo(int hue, int saturation,
+				   int brightness, int sharpness,
+				   int noise_reduction, int contrast,
 				   int overscan, int vo_aspect_ratio)
 {
-  char cmd[128];
   Xine_Control("OVERSCAN", overscan);
-  snprintf(cmd, sizeof(cmd),
-	   "VIDEO_PROPERTIES %d %d %d %d %d", 
-	  hue, saturation, brightness, contrast, vo_aspect_ratio);
-  return Xine_Control(cmd);
+  return Xine_Control(cString::sprintf("VIDEO_PROPERTIES %d %d %d %d %d %d %d",
+                                       hue, saturation, brightness, sharpness,
+                                       noise_reduction, contrast, vo_aspect_ratio));
 }
 
 //
