@@ -130,6 +130,7 @@ cXinelibServer::cXinelibServer(cXinelibDevice *Dev, int listen_port) :
     m_bMulticast[i] = 0;
     m_bConfigOk[i] = false;
     m_bArgbOSD[i] = false;
+    m_bRleArgbOSD[i] = false;
     m_bUdp[i] = 0;
     m_ConnType[i] = ctDetecting;
   }
@@ -227,6 +228,7 @@ void cXinelibServer::CloseDataConnection(int cli)
   m_bMulticast[cli] = false;
   m_bConfigOk[cli] = false;
   m_bArgbOSD[cli] = false;
+  m_bRleArgbOSD[cli] = false;
 
   m_iMulticastMask &= ~(1<<cli);
 
@@ -314,7 +316,9 @@ void cXinelibServer::OsdCmd(void *cmd_gen)
     void *compressed_data = NULL;
     osd_command_t *cmd = (osd_command_t*)cmd_gen;
     osd_command_t cmdnet;
+    osd_command_t cmdnet_argbrle;
     memcpy(&cmdnet, cmd, sizeof(osd_command_t));
+    memset(&cmdnet_argbrle, 0, sizeof(cmdnet_argbrle));
 
     if (cmd->data) {
       // compress data
@@ -329,6 +333,17 @@ void cXinelibServer::OsdCmd(void *cmd_gen)
         cmdnet.raw_data = (uint8_t *)malloc(cmd->datalen);
         cmdnet.datalen  = rle_recompress_net(cmdnet.raw_data, cmd->data, cmd->num_rle);
         compressed_data = cmdnet.raw_data; // free it later
+
+      } else if (cmd->cmd == OSD_Set_ARGB) {
+        int num_rle = 0;
+        memcpy(&cmdnet_argbrle, cmd, sizeof(osd_command_t));
+        cmdnet_argbrle.datalen  = rle_compress_argbrle(&cmdnet_argbrle.raw_data,
+                                                    (const uint32_t *)cmd->raw_data,
+                                                    cmd->w, cmd->h, &num_rle);
+        cmdnet_argbrle.num_rle  = num_rle;
+        cmdnet_argbrle.cmd      = OSD_Set_ARGBRLE;
+        compressed_data         = cmdnet_argbrle.raw_data; // free it later
+        hton_osdcmd(cmdnet_argbrle);
       }
     }
 
@@ -337,7 +352,13 @@ void cXinelibServer::OsdCmd(void *cmd_gen)
 
     for(i = 0; i < MAXCLIENTS; i++) {
       if(fd_control[i].open() && m_bConfigOk[i]) {
-        int r = write_osd_command(fd_control[i], &cmdnet);
+        int r;
+        if (m_bRleArgbOSD[i] && cmdnet_argbrle.data) {
+          r = write_osd_command(fd_control[i], &cmdnet_argbrle);
+        } else {
+          r = write_osd_command(fd_control[i], &cmdnet);
+        }
+
         if(r < 0) {
           LOGMSG("Send OSD command failed, closing connection");
           CloseConnection(i);
@@ -1530,24 +1551,22 @@ void cXinelibServer::Handle_Control_RTSP(int cli, const char *arg)
         if (fd_control[cli].getsockname((struct sockaddr *)&sin, &len) < 0) {
           LOGERR("Error getting control socket address");
         }
-        const char *sdp_descr = vdr_sdp_description(cxSocket::ip2txt(sin.sin_addr.s_addr,
-                                                                     sin.sin_port, buf),
-                                                    2001,
-                                                    xc.listen_port,
-                                                    xc.remote_rtp_addr,
-                                                    payload_type,
-                                                    /*m_ssrc*/0x4df73452,
-                                                    xc.remote_rtp_port,
-                                                    xc.remote_rtp_ttl);
-        size_t sdplen = sdp_descr ? strlen(sdp_descr) : 0;
+        cString sdp_descr = vdr_sdp_description(cxSocket::ip2txt(sin.sin_addr.s_addr,
+                                                                 sin.sin_port, buf),
+                                                2001,
+                                                xc.listen_port,
+                                                xc.remote_rtp_addr,
+                                                payload_type,
+                                                /*m_ssrc*/0x4df73452,
+                                                xc.remote_rtp_port,
+                                                xc.remote_rtp_ttl);
+        size_t sdplen = strlen(sdp_descr);
         RTSPOUT(RTSP_200_OK
                 "Content-Type: application/sdp\r\n"
-                "Content-Length: %lu\r\n"
+                "Content-Length: %zu\r\n"
                 "\r\n",
-                CSeq, (unsigned long)sdplen);
-        if (sdplen) {
-          fd_control[cli].write_cmd(sdp_descr, sdplen);
-        }
+                CSeq, sdplen);
+        fd_control[cli].write_cmd(sdp_descr, sdplen);
       } else {
         RTSPOUT(RTSP_415 /*UNSUPPORTED_MEDIATYPE*/);
       }
@@ -1706,6 +1725,9 @@ void cXinelibServer::Handle_Control(int cli, const char *cmd)
     if(!strncmp(cmd, "INFO ARGBOSD", 12))
       m_bArgbOSD[cli] = true;
 
+    if(!strncmp(cmd, "INFO ARGBOSD RLE", 16))
+      m_bRleArgbOSD[cli] = true;
+
     if(!*xc.local_frontend || !strncmp(xc.local_frontend, "none", 4))
       cXinelibThread::InfoHandler(cmd+5);
 
