@@ -104,8 +104,6 @@ class cXinelibOsd : public cOsd, public cListObject
                  int W, int H, unsigned char *Data,
                  int Colors, unsigned int *Palette,
                  osd_rect_t *DirtyArea);
-    void CmdPalette(int Wnd, int Colors, unsigned int *Palette);
-    void CmdMove(int Wnd, int Width, int Height);
     void CmdClose(int Wnd);
     void CmdFlush(void);
 
@@ -194,7 +192,7 @@ void cXinelibOsd::CmdSize(int Width, int Height)
   TRACEF("cXinelibOsd::CmdSize");
 
   if (m_Device && m_WindowHandles) {
-    osd_command_t osdcmd = {0};
+    osd_command_t osdcmd = {};
 
     osdcmd.cmd = OSD_Size;
     osdcmd.w   = Width;
@@ -216,7 +214,7 @@ void cXinelibOsd::CmdVideoWindow(int X, int Y, int W, int H)
   TRACEF("cXinelibOsd::CmdVideoWindow");
 
   if (m_Device && m_WindowHandles) {
-    osd_command_t osdcmd = {0};
+    osd_command_t osdcmd = {};
 
     for (int Wnd = 0; m_WindowHandles[Wnd] >= 0; Wnd++) {
       osdcmd.cmd = OSD_VideoWindow;
@@ -231,47 +229,12 @@ void cXinelibOsd::CmdVideoWindow(int X, int Y, int W, int H)
   }
 }
 
-void cXinelibOsd::CmdMove(int Wnd, int NewX, int NewY)
-{
-  TRACEF("cXinelibOsd::CmdMove");
-
-  if (m_Device && m_WindowHandles) {
-    osd_command_t osdcmd = {0};
-
-    osdcmd.cmd = OSD_Move;
-    osdcmd.wnd = m_WindowHandles[Wnd];
-    osdcmd.x   = NewX;
-    osdcmd.y   = NewY;
-
-    m_Device->OsdCmd((void*)&osdcmd);
-  }
-}
-
-void cXinelibOsd::CmdPalette(int Wnd, int Colors, unsigned int *Palette)
-{
-  TRACEF("cXinelibOsd::CmdPalette");
-
-  if (m_Device) {
-    osd_clut_t    clut[Colors];
-    osd_command_t osdcmd = {0};
-
-    osdcmd.cmd     = OSD_SetPalette;
-    osdcmd.wnd     = m_WindowHandles[Wnd];
-    osdcmd.palette = clut;
-    osdcmd.colors  = Colors;
-
-    prepare_palette(&clut[0], Palette, Colors, /*Top*/(Prev() == NULL), true);
-
-    m_Device->OsdCmd((void*)&osdcmd);
-  }
-}
-
 void cXinelibOsd::CmdClose(int Wnd)
 {
   TRACEF("cXinelibOsd::CmdClose");
 
   if (m_Device) {
-    osd_command_t osdcmd = {0};
+    osd_command_t osdcmd = {};
 
     osdcmd.cmd = OSD_Close;
     osdcmd.wnd = m_WindowHandles[Wnd];
@@ -290,7 +253,7 @@ void cXinelibOsd::CmdFlush(void)
   TRACEF("cXinelibOsd::CmdFlush");
 
   if (m_Device) {
-    osd_command_t osdcmd = {0};
+    osd_command_t osdcmd = {};
 
     osdcmd.cmd = OSD_Flush;
 
@@ -322,7 +285,7 @@ void cXinelibOsd::CmdArgb(int X, int Y, int W, int H,
 
   if (m_Device) {
 
-    osd_command_t osdcmd = {0};
+    osd_command_t osdcmd = {};
 
     osdcmd.cmd   = OSD_Set_ARGB;
     osdcmd.wnd   = m_WindowHandles[0];
@@ -355,7 +318,7 @@ void cXinelibOsd::CmdLut8(int Wnd, int X0, int Y0,
   if (m_Device) {
 
     osd_clut_t    clut[Colors];
-    osd_command_t osdcmd = {0};
+    osd_command_t osdcmd = {};
 
     osdcmd.cmd   = OSD_Set_LUT8;
     osdcmd.wnd   = m_WindowHandles[Wnd];
