@@ -21,7 +21,6 @@
 #include <netinet/tcp.h> // CORK, NODELAY
 
 #include <vdr/tools.h>
-#include <vdr/config.h>  // VDRVERSNUM
 
 #include "../logdefs.h"
 #include "../xine_input_vdr_net.h" // stream_tcp_header_t
@@ -259,7 +258,7 @@ int cTcpWriter::Put(eStreamId StreamId, uint64_t StreamPos,
                     const uchar *Data, int DataCount)
 {
   stream_tcp_header_t header;
-  header.pos = htonull(StreamPos);
+  header.pos = priv_htonull(StreamPos);
   header.len = htonl(DataCount);
   header.stream = (uint8_t)StreamId;
   return Put((uchar*)&header, sizeof(header), Data, DataCount);
