@@ -285,9 +285,7 @@ int ts_parse_pmt (pmt_data_t *pmt, unsigned program_no, const uint8_t *pkt)
     pkt += pkt[4]; /* pointer */
     offset = 1;
 
-    if (pmt->pmt != NULL) 
-      free(pmt->pmt);
-    pmt->pmt = (uint8_t *) calloc(4096, sizeof(uint8_t));
+    pmt->pmt = pmt->raw_pmt;
     pmt->pmt_write_ptr = pmt->pmt;
 
     section_syntax_indicator  = (pkt[6] >> 7) & 0x01;
@@ -346,7 +344,6 @@ int ts_parse_pmt (pmt_data_t *pmt, unsigned program_no, const uint8_t *pkt)
   }
 
   if (!section_length) {
-    free(pmt->pmt);
     pmt->pmt = NULL;
     LOGMSG("parse_pmt: zero-length section");
     return 0;
@@ -386,7 +383,7 @@ int ts_parse_pmt (pmt_data_t *pmt, unsigned program_no, const uint8_t *pkt)
   ts_get_reg_desc(&program_info_format_identifier,
                   &pmt->pmt[12], program_info_length);
   if (program_info_format_identifier) {
-    if ((program_info_format_identifier == (('H' << 24) | ('D' << 16) | ('M' << 8) | 'V'))) {
+    if (program_info_format_identifier == (('H' << 24) | ('D' << 16) | ('M' << 8) | 'V')) {
       LOGMSG("PMT program info has tag 0x05 (format_identifier), content HDMV (0x%x)\n", program_info_format_identifier);
       hdmv_pmt = 1;
     } else {
@@ -772,13 +769,13 @@ int64_t ts_get_pts(ts_state_t *ts, const uint8_t *data)
 /*
  * ts_get_picture_type()
  */
-
+/*
 int ts_get_picture_type(ts_state_t *ts, const uint8_t *data, int h264)
 {
   int pic = NO_PICTURE;
   return pic;
 }
-
+*/
 /*
  * ts_get_video_size()
  */
