@@ -13,8 +13,6 @@
 
 #include <stdint.h>
 
-#include <vdr/config.h>  // VDRVERSNUM
-
 #include "../xine_input_vdr_net.h" // frame headers
 #include "ts.h"
 
@@ -111,14 +109,10 @@ class cUdpBackLog
 
       // RTP header
       header->rtp_hdr.raw[0] = RTP_VERSION_BYTE | RTP_HDREXT_BIT;
-#if VDRVERSNUM >= 10701 || defined(TSPLAY_PATCH_VERSION)
       if (DATA_IS_TS(Data))
         header->rtp_hdr.raw[1] = RTP_PAYLOAD_TYPE_TS;
       else
         header->rtp_hdr.raw[1] = RTP_PAYLOAD_TYPE_PES;
-#else
-      header->rtp_hdr.raw[1] = RTP_PAYLOAD_TYPE_PES;
-#endif
       header->rtp_hdr.seq  = htons(m_RtpSeqNo & 0xFFFF);
       /*header->rtp_hdr.ts   = htonl((uint32_t)(RtpScr.Now() & 0xffffffff));*/
       /*header->rtp_hdr.ssrc = htonl(m_ssrc);*/
@@ -128,7 +122,7 @@ class cUdpBackLog
       header->hdr_ext.hdr.type = htons(RTP_HEADER_EXT_X_TYPE);
 
       // UDP header
-      header->hdr_ext.pos = htonull(StreamPos);
+      header->hdr_ext.pos = priv_htonull(StreamPos);
       header->hdr_ext.seq = htons(m_SeqNo);
       header->hdr_ext.stream = (uint8_t)StreamId;
 
