@@ -15,13 +15,17 @@
 #  define _GNU_SOURCE  /* asprintf */
 #endif
 
+#include <stdint.h>
 #include <stdlib.h>
 #include <stdio.h>
 #ifdef __FreeBSD__
 #include <string.h>
 #endif
-#include <poll.h>
 #include <unistd.h>
+#include <sys/select.h>
+#include <sys/time.h>
+#include <sys/types.h>
+
 #include <netinet/in.h>
 #include <sys/socket.h>
 #include <sys/un.h>
@@ -160,14 +164,19 @@ int udp_discovery_recv(int fd_discovery, char *buf, int timeout,
 		       struct sockaddr_in *source)
 {
   socklen_t sourcelen = sizeof(struct sockaddr_in);
-  struct pollfd pfd;
   int err;
 
-  pfd.fd = fd_discovery;
-  pfd.events = POLLIN;
+  struct timeval select_timeout;
+  fd_set fdset;
+
+  FD_ZERO (&fdset);
+  FD_SET  (fd_discovery, &fdset);
+
+  select_timeout.tv_sec  = 0;
+  select_timeout.tv_usec = 1000 * timeout;
+
+  err = select (fd_discovery + 1, &fdset, NULL, NULL, &select_timeout);
 
-  errno = 0;
-  err = poll(&pfd, 1, timeout);
   if(err < 1) {
     if(err < 0)
       LOGERR("broadcast poll error");
@@ -199,21 +208,14 @@ int udp_discovery_is_valid_search(const char *buf)
   return 0;
 }
 
-int udp_discovery_find_server(int *port, char *address)
+static int _udp_discovery_find_server(int fd_discovery, int *port, char *address)
 {
   static const char mystring[] = DISCOVERY_1_0_HDR "Server port: ";
   struct sockaddr_in from;
   char buf[DISCOVERY_MSG_MAXSIZE];
-  int fd_discovery = -1;
   int trycount = 0;
   int err = 0;
 
-  *port = DISCOVERY_PORT;
-  strcpy(address, "vdr");
-
-  if((fd_discovery = discovery_init(DISCOVERY_PORT)) < 0)
-    return 0;
-
   while(err >= 0 && ++trycount < 4) {
 
     if((err = udp_discovery_search(fd_discovery, DISCOVERY_PORT) >= 0)) {
@@ -262,7 +264,6 @@ int udp_discovery_find_server(int *port, char *address)
 	  *port = -1;
           if(1 == sscanf(buf + strlen(mystring), "%d", port) &&
              *port >= 1000 && *port <= 0xffff) {
-            close(fd_discovery);
 	    return 1;
           }
 	  LOGMSG("Server-given port is invalid !");
@@ -274,8 +275,21 @@ int udp_discovery_find_server(int *port, char *address)
   }
 
   /* failed */
-  close(fd_discovery);
   return 0;
 }
 
+int udp_discovery_find_server(int *port, char *address)
+{
+  int fd_discovery;
+  int ret = 0;
 
+  *port = DISCOVERY_PORT;
+  strcpy(address, "vdr");
+
+  if((fd_discovery = discovery_init(DISCOVERY_PORT)) >= 0) {
+    ret = _udp_discovery_find_server(fd_discovery, port, address);
+    close(fd_discovery);
+  }
+
+  return ret;
+}
