@@ -528,10 +528,10 @@ static int exec_osd_set_rle(osd_manager_impl_t *this, osd_command_t *cmd)
 
   /* fill extra data */
   const vdr_osd_extradata_t extra_data = {
-    extent_width:  osd->extent_width,
-    extent_height: osd->extent_height,
-    layer:         cmd->layer,
-    scaling:       cmd->scaling
+    .extent_width  = osd->extent_width,
+    .extent_height = osd->extent_height,
+    .layer         = cmd->layer,
+    .scaling       = cmd->scaling
   };
   memcpy(ov_overlay.hili_color, &extra_data, sizeof(extra_data));
 
@@ -650,10 +650,10 @@ static int exec_osd_set_argb(osd_manager_impl_t *this, osd_command_t *cmd)
 
   /* fill extra data */
   const vdr_osd_extradata_t extra_data = {
-    extent_width:  osd->extent_width,
-    extent_height: osd->extent_height,
-    layer:         cmd->layer,
-    scaling:       cmd->scaling
+    .extent_width  = osd->extent_width,
+    .extent_height = osd->extent_height,
+    .layer         = cmd->layer,
+    .scaling       = cmd->scaling
   };
   memcpy(ov_overlay.hili_color, &extra_data, sizeof(extra_data));
 
@@ -703,12 +703,20 @@ static int exec_osd_set_argb(osd_manager_impl_t *this, osd_command_t *cmd)
 
   /* blend */
   dst += cmd->y * osd->extent_width + cmd->x;
-  for (; lines > 0; lines--) {
-    memcpy(dst, src, stride * sizeof(uint32_t));
-    src += cmd->w;
-    dst += osd->extent_width;
+  if (cmd->cmd == OSD_Set_ARGBRLE) {
+    int res = rle_uncompress_argbrle(dst, cmd->w, lines, osd->extent_width,
+                                     cmd->raw_data, cmd->num_rle,
+                                     cmd->datalen);
+    if (res < 0)
+      LOGMSG("ARGB uncompress failed: %d", res);
+    /* ignore failure, can be caused by cropping */
+  } else {
+    for (; lines > 0; lines--) {
+      memcpy(dst, src, stride * sizeof(uint32_t));
+      src += cmd->w;
+      dst += osd->extent_width;
+    }
   }
-
   /* set dirty area. not used in opengl2 driver ... */
 #if 0
   /* this can't work... There's no way to know if driver has
@@ -750,83 +758,6 @@ static int exec_osd_set_argb(osd_manager_impl_t *this, osd_command_t *cmd)
 #endif
 }
 
-/*
- * exec_osd_set_palette()
- *
- * - replace palette of an already existing OSD
- */
-static int exec_osd_set_palette(osd_manager_impl_t *this, osd_command_t *cmd)
-{
-  osd_data_t *osd = &this->osd[cmd->wnd];
-
-  if (!osd->cmd.data) {
-    LOGMSG("OSD_SetPalette(%d): old RLE data missing !", cmd->wnd);
-    return CONTROL_PARAM_ERROR;
-  }
-  if (!cmd->palette) {
-    LOGMSG("OSD_SetPalette(%d): new palette missing !", cmd->wnd);
-    return CONTROL_PARAM_ERROR;
-  }
-
-  /* use cached event to re-create Set_RLE command with modified palette */
-  osd_command_t tmp;
-  /* steal the original command */
-  memcpy(&tmp, &osd->cmd, sizeof(osd_command_t));
-  memset(&osd->cmd, 0, sizeof(osd_command_t));
-
-  /* replace palette */
-  tmp.cmd      = OSD_Set_RLE;
-  free(tmp.palette);
-  tmp.palette  = malloc(cmd->colors * sizeof(osd_rle_elem_t));
-  memcpy(tmp.palette, cmd->palette, cmd->colors * sizeof(osd_rle_elem_t));
-  tmp.colors   = cmd->colors;
-  tmp.pts      = cmd->pts;
-  tmp.delay_ms = cmd->delay_ms;
-  tmp.flags   |= cmd->flags & OSDFLAG_YUV_CLUT;
-
-  /* redraw */
-  int r = exec_osd_set_rle(this, &tmp);
-  clear_osdcmd(&tmp);
-  return r;
-}
-
-/*
- * exec_osd_move()
- *
- * - move existing OSD to new position
- */
-static int exec_osd_move(osd_manager_impl_t *this, osd_command_t *cmd)
-{
-  osd_data_t *osd = &this->osd[cmd->wnd];
-
-  if (!osd->cmd.data) {
-    LOGMSG("OSD_Move(%d): old RLE data missing !", cmd->wnd);
-    return CONTROL_PARAM_ERROR;
-  }
-  if (!osd->cmd.palette) {
-    LOGMSG("OSD_Move(%d): old palette missing !", cmd->wnd);
-    return CONTROL_PARAM_ERROR;
-  }
-
-  /* use cached event to re-create Set_RLE command with modified palette */
-  osd_command_t tmp;
-  /* steal the original command */
-  memcpy(&tmp, &osd->cmd, sizeof(osd_command_t));
-  memset(&osd->cmd, 0, sizeof(osd_command_t));
-
-  /* replace position */
-  tmp.cmd      = OSD_Set_RLE;
-  tmp.x        = cmd->x;
-  tmp.y        = cmd->y;
-  tmp.pts      = cmd->pts;
-  tmp.delay_ms = cmd->delay_ms;
-
-  /* redraw */
-  int r = exec_osd_set_rle(this, &tmp);
-  clear_osdcmd(&tmp);
-  return r;
-}
-
 /*
  * exec_osd_command_internal()
  */
@@ -837,11 +768,10 @@ static int exec_osd_command_internal(osd_manager_impl_t *this, struct osd_comman
   switch (cmd->cmd) {
   case OSD_Nop:        return exec_osd_nop(this, cmd);
   case OSD_Size:       return exec_osd_size(this, cmd);
-  case OSD_SetPalette: return exec_osd_set_palette(this, cmd);
-  case OSD_Move:       return exec_osd_move(this, cmd);
   case OSD_Flush:      return exec_osd_flush(this, cmd);
   case OSD_Set_RLE:    return exec_osd_set_rle(this, cmd);
   case OSD_Set_LUT8:   return exec_osd_set_lut8(this, cmd);
+  case OSD_Set_ARGBRLE:
   case OSD_Set_ARGB:   return exec_osd_set_argb(this, cmd);
   case OSD_Close:      return exec_osd_close(this, cmd);
   case OSD_VideoWindow:return exec_osd_video_window(this, cmd);
@@ -849,10 +779,6 @@ static int exec_osd_command_internal(osd_manager_impl_t *this, struct osd_comman
     /* All OSD areas have been updated, commit changes to display */
     /* - not used with traditional xine-lib OSD */
     return CONTROL_OK;
-  case OSD_Set_YUV:
-    /* TODO */
-    LOGMSG("OSD_Set_YUV not implemented !");
-    return CONTROL_PARAM_ERROR;
   default:
     LOGMSG("Unknown OSD command %d", cmd->cmd);
     return CONTROL_PARAM_ERROR;
