@@ -25,7 +25,9 @@
 # ifdef boolean
 #  define HAVE_BOOLEAN
 # endif
+# define INT32 jpegINT32
 # include <jpeglib.h>
+# undef INT32
 # undef boolean
 #endif
 
@@ -147,10 +149,42 @@ static void make_dirs(const char *file)
  * list available plugins
  */
 
-static void list_plugins_type(xine_t *xine, const char *msg, typeof (xine_list_audio_output_plugins) list_func)
+static int plugins_check(xine_t *xine)
+{
+  const char *const *list;
+  int basic = 0, advanced = 0;
+
+  list = xine_list_video_decoder_plugins(xine);
+  while (list && *list) {
+    if (!strcmp(*list,  "ffmpegvideo") ||
+        !strncmp(*list, "vdpau_", 6) ||
+        !strcmp(*list,  "libmmal")) {
+      advanced = 1;
+      break;
+    }
+    if (!strcmp(*list,  "mpeg2")) {
+      basic = 1;
+    }
+    list++;
+  }
+
+  if (!advanced) {
+    if (!basic) {
+      LOGERR("FATAL: Required xine video decoder plugins are missing! Please install plugins.");
+      return -1;
+    } else {
+      LOGMSG("WARNING: Essential xine video decoder plugins not found! Please install plugins to display H.264 video.");
+    }
+  }
+
+  return 0;
+}
+
+#define LIST_FUNC __typeof__(xine_list_audio_output_plugins)
+
+static void list_plugins_type(xine_t *xine, const char *msg, LIST_FUNC list_func)
 {
   const char *const *list = list_func(xine);
-
   printf("%s", msg);
   while(list && *list)
     printf(" %s", *list++);
@@ -459,7 +493,7 @@ static void configure_audio_out(const fe_t *this, const char *audio_driver, cons
     
     /* define possible speaker arrangements */
     /* From xine audio_alsa_out.c ; must be synchronized ! */
-    static char *speaker_arrangement[] = 
+    static const char * const speaker_arrangement[] = 
       {"Mono 1.0", "Stereo 2.0", "Headphones 2.0", "Stereo 2.1",
        "Surround 3.0", "Surround 4.0", "Surround 4.1", 
        "Surround 5.0", "Surround 5.1", "Surround 6.0",
@@ -467,7 +501,7 @@ static void configure_audio_out(const fe_t *this, const char *audio_driver, cons
 
     x_reg_enum("audio.output.speaker_arrangement",
 	       STEREO,
-	       speaker_arrangement,
+	       (char**)(intptr_t)speaker_arrangement,
 	       _("speaker arrangement"),
 	       _("Select how your speakers are arranged, "
 		 "this determines which speakers xine uses for sound output. "
@@ -608,13 +642,23 @@ static int fe_xine_init(frontend_t *this_gen, const char *audio_driver,
   this->audio_port      = NULL;
   this->input_plugin    = NULL;
 
+  /* Set log level for input plugin.
+   * Also inform input plugin we're not running in xine-ui etc.
+   */
+  static char log_level[16];
+  sprintf(log_level, "%d", SysLogLevel);
+  setenv("VDR_FE_LOG_LEVEL", log_level, 1);
+  if (LogToSysLog) {
+    setenv("VDR_FE_SYSLOG", "1", 1);
+  }
+
   /* create a new xine and load config file */
   this->xine = xine_new();
   if(!this->xine)
     return 0;
 
   /* Set xine engine logging verbosity */
-  this->xine->verbosity = (SysLogLevel>2);
+  this->xine->verbosity = (SysLogLevel>2) ? (SysLogLevel - 1) : 0;
 
   /*xine_register_log_cb(this->xine, xine_log_cb, this);*/
 
@@ -650,6 +694,9 @@ static int fe_xine_init(frontend_t *this_gen, const char *audio_driver,
 
   xine_init (this->xine);
 
+  if (plugins_check(this->xine) < 0)
+    return 0;
+
   x_upd_num("video.device.xv_double_buffer", 1);
   x_upd_num("engine.buffers.video_num_buffers", pes_buffers);
 
@@ -660,27 +707,53 @@ static int fe_xine_init(frontend_t *this_gen, const char *audio_driver,
 
   this->playback_finished = 0;
 
+  /* check X11 video driver type */
+  if (this->xine_visual_type == XINE_VISUAL_TYPE_X11) {
+    if (video_driver &&
+        ( !strcasecmp(video_driver, "xshm") ||
+          !strcasecmp(video_driver, "xv") ||
+          !strcasecmp(video_driver, "xcbshm") ||
+          !strcasecmp(video_driver, "xcbxv") ||
+          !strcasecmp(video_driver, "opengl") ||
+          !strcasecmp(video_driver, "xvmc") ||
+          !strcasecmp(video_driver, "xxmc"))) {
+      LOGMSG("WARNING: Using inefficient legacy video driver %s\n", video_driver);
+    }
+    if (!video_driver) {
+      LOGMSG("WARNING: no video driver given. Trying OpenGL2.\n");
+      this->video_port = xine_open_video_driver(this->xine,
+                                                "opengl2",
+                                                this->xine_visual_type,
+                                                (void *) &(this->vis));
+      if (!this->video_port)
+        LOGMSG("Failed initializing OpenGL2 video output.\n");
+    }
+  }
+
   /* create video port */
-  if(video_driver && !strcmp(video_driver, "none")) 
-    this->video_port = xine_open_video_driver(this->xine,
-					      video_driver,
-					      XINE_VISUAL_TYPE_NONE, 
-					      NULL);
-  else if(video_driver && !strcmp(video_driver, "dxr3"))
-    this->video_port = xine_open_video_driver(this->xine,
-					      video_driver,
-					      XINE_VISUAL_TYPE_X11, 
-					      NULL);
-  else if(video_driver && !strcmp(video_driver, "aadxr3"))
-    this->video_port = xine_open_video_driver(this->xine,
-					      video_driver,
-					      XINE_VISUAL_TYPE_AA, 
-					      NULL);
-  else
-    this->video_port = xine_open_video_driver(this->xine,
-					      video_driver,
-					      this->xine_visual_type,
-					      (void *) &(this->vis));
+  if (!this->video_port) {
+    if (video_driver && !strcmp(video_driver, "none"))
+      this->video_port = xine_open_video_driver(this->xine,
+                                                video_driver,
+                                                XINE_VISUAL_TYPE_NONE,
+                                                NULL);
+    else if (video_driver && !strcmp(video_driver, "dxr3"))
+      this->video_port = xine_open_video_driver(this->xine,
+                                                video_driver,
+                                                XINE_VISUAL_TYPE_X11,
+                                                NULL);
+    else if(video_driver && !strcmp(video_driver, "aadxr3"))
+      this->video_port = xine_open_video_driver(this->xine,
+                                                video_driver,
+                                                XINE_VISUAL_TYPE_AA,
+                                                NULL);
+    else
+      this->video_port = xine_open_video_driver(this->xine,
+                                                video_driver,
+                                                this->xine_visual_type,
+                                                (void *) &(this->vis));
+  }
+
   if(!this->video_port) {
     LOGMSG("fe_xine_init: xine_open_video_driver(\"%s\") failed",
 	   video_driver?video_driver:"(NULL)"); 
@@ -1224,7 +1297,7 @@ static void fe_xine_close(frontend_t *this_gen)
     fe_xine_stop(this_gen);
 
     xine_close(this->stream);
-    if(this->postplugins->pip_stream) 
+    if (this->postplugins->pip_stream)
       xine_close(this->postplugins->pip_stream);
   }
 }
@@ -1475,8 +1548,8 @@ static void *fe_control(frontend_t *this_gen, const char *cmd)
   /*LOGDBG("fe_control(\"%s\")", cmd);*/
 
   if(!cmd || !this) {
-    LOGMSG("fe_control(0x%lx,0x%lx) : invalid argument", 
-	   (unsigned long int)this_gen, (unsigned long int)cmd);
+    LOGMSG("fe_control(%p,%p) : invalid argument",
+           (const void *)this_gen, (const void *)cmd);
     return NULL;
   }
 
@@ -1494,9 +1567,9 @@ static void *fe_control(frontend_t *this_gen, const char *cmd)
     init_dummy_ports(this, 0);
     this->video_width = this->video_height = 0;
 
-  } else if(!strncmp(cmd, "SLAVE 0x", 8)) {
-    unsigned long pt;
-    if(1 == sscanf(cmd+8, "%lx", &pt)) {
+  } else if(!strncmp(cmd, "SLAVE ", 6)) {
+    void *pt;
+    if(1 == sscanf(cmd+6, "%p", &pt)) {
       xine_stream_t *slave_stream = (xine_stream_t*)pt;
       if(this->slave_stream != slave_stream) {
 
@@ -1545,8 +1618,8 @@ static void *fe_control(frontend_t *this_gen, const char *cmd)
 					    this->video_port);
       LOGMSG("  PIP %d: %dx%d @ (%d,%d)", pid & 0x0f, w, h, x, y);
       LOGMSG("create pip stream done");
-      sprintf(mrl, MRL_ID "+slave://0x%lx#nocache",
-	      (unsigned long int)this);
+      sprintf(mrl, MRL_ID "+slave://%p#nocache",
+	      (void *)this);
       if(!xine_open(posts->pip_stream, mrl) ||
 	 !xine_play(posts->pip_stream, 0, 0)) {
 	LOGERR("  pip stream open/play failed");
@@ -1913,7 +1986,7 @@ static char *frame_compress_pnm(fe_t *this, int *size, vo_frame_t *frame)
 #endif /* HAVE_XINE_GRAB_VIDEO_FRAME */
 
 #ifdef HAVE_XINE_GRAB_VIDEO_FRAME
-static char *fe_compress_grab_frame(fe_t *this, int *size, int jpeg, int quality, int width, int height, xine_grab_video_frame_t *frame)
+static char *fe_compress_grab_frame(int *size, int jpeg, int quality, int width, int height, xine_grab_video_frame_t *frame)
 {
 #ifdef HAVE_LIBJPEG
   if (jpeg) {
@@ -2011,7 +2084,7 @@ static char *fe_grab(frontend_t *this_gen, int *size, int jpeg,
     grab_frame->width = width;
     grab_frame->height = height;
     if (!grab_frame->grab(grab_frame))
-      img = fe_compress_grab_frame(this, size, jpeg, quality, width, height, grab_frame);
+      img = fe_compress_grab_frame(size, jpeg, quality, width, height, grab_frame);
     grab_frame->dispose(grab_frame);
   }
   this->stream->xine->port_ticket->release(this->stream->xine->port_ticket, 0);
